@@ -5,6 +5,7 @@ from ._fc import FullyConnectedNet
 from ._gate import Gate
 from ._identity import Identity
 from ._s2act import S2Activation
+from ._so3act import SO3Activation
 from ._normact import NormActivation
 from ._dropout import Dropout
 
@@ -18,6 +19,7 @@ __all__ = [
     "Gate",
     "Identity",
     "S2Activation",
+    "SO3Activation",
     "NormActivation",
     "Dropout",
 ]
