@@ -1,12 +1,13 @@
 import itertools
 import collections
-import warnings
-from typing import List
+from typing import List, Union
 
 import torch
 
-from e3nn import o3
 from e3nn.math import direct_sum, perm
+# These imports avoid cyclic reference from o3 itself
+from . import _rotation
+from . import _wigner
 
 
 class Irrep(tuple):
@@ -49,7 +50,7 @@ class Irrep(tuple):
     >>> Irrep("1o") + Irrep("2o")
     1x1o+1x2o
     """
-    def __new__(cls, l, p=None):
+    def __new__(cls, l: Union[int, 'Irrep', str, tuple], p=None):
         if p is None:
             if isinstance(l, Irrep):
                 return l
@@ -141,7 +142,7 @@ class Irrep(tuple):
             k = torch.zeros_like(alpha)
 
         alpha, beta, gamma, k = torch.broadcast_tensors(alpha, beta, gamma, k)
-        return o3.wigner_D(self.l, alpha, beta, gamma) * self.p**k[..., None, None]
+        return _wigner.wigner_D(self.l, alpha, beta, gamma) * self.p**k[..., None, None]
 
     def D_from_quaternion(self, q, k=None):
         r"""Matrix of the representation, see `Irrep.D_from_angles`
@@ -159,7 +160,7 @@ class Irrep(tuple):
         `torch.Tensor`
             tensor of shape :math:`(..., 2l+1, 2l+1)`
         """
-        return self.D_from_angles(*o3.quaternion_to_angles(q), k)
+        return self.D_from_angles(*_rotation.quaternion_to_angles(q), k)
 
     def D_from_matrix(self, R):
         r"""Matrix of the representation, see `Irrep.D_from_angles`
@@ -188,19 +189,23 @@ class Irrep(tuple):
         d = torch.det(R).sign()
         R = d[..., None, None] * R
         k = (1 - d) / 2
-        return self.D_from_angles(*o3.matrix_to_angles(R), k)
+        return self.D_from_angles(*_rotation.matrix_to_angles(R), k)
 
     @property
     def dim(self) -> int:
         """The dimension of the representation, :math:`2 l + 1`."""
         return 2 * self.l + 1
 
+    def is_scalar(self) -> bool:
+        """Equivalent to ``l == 0 and p == 1``"""
+        return self.l == 0 and self.p == 1
+
     def __mul__(self, other):
         r"""Generate the irreps from the product of two irreps.
 
         Returns
         -------
-        generator of `Irrep`
+        generator of `e3nn.o3.Irrep`
         """
         other = Irrep(other)
         p = self.p * other.p
@@ -247,7 +252,7 @@ class _MulIr(tuple):
         return self[0]
 
     @property
-    def ir(self) -> int:
+    def ir(self) -> Irrep:
         return self[1]
 
     @property
@@ -314,7 +319,7 @@ class Irreps(tuple):
     >>> Irreps(), Irreps("")
     (, )
     """
-    def __new__(cls, irreps=None):
+    def __new__(cls, irreps=None) -> Union[_MulIr, 'Irreps']:
         if isinstance(irreps, Irreps):
             return super().__new__(cls, irreps)
 
@@ -341,6 +346,9 @@ class Irreps(tuple):
             pass
         else:
             for mul_ir in irreps:
+                mul = None
+                ir = None
+
                 if isinstance(mul_ir, str):
                     mul = 1
                     ir = Irrep(mul_ir)
@@ -352,13 +360,6 @@ class Irreps(tuple):
                 elif len(mul_ir) == 2:
                     mul, ir = mul_ir
                     ir = Irrep(ir)
-                elif len(mul_ir) == 3:
-                    mul, l, p = mul_ir
-                    ir = Irrep(l, p)
-                    warnings.warn("Use the argument format [(mul, (l, p)), ...] to distinguish multiplicity from irrep", DeprecationWarning, stacklevel=2)
-                else:
-                    mul = None
-                    ir = None
 
                 if not (isinstance(mul, int) and mul >= 0 and ir is not None):
                     raise ValueError(f"Unable to interpret \"{mul_ir}\" as an irrep.")
@@ -380,7 +381,7 @@ class Irreps(tuple):
 
         Returns
         -------
-        `Irreps`
+        `e3nn.o3.Irreps`
             representation of :math:`(Y^0, Y^1, \dots, Y^{\mathrm{lmax}})`
 
         Examples
@@ -452,7 +453,7 @@ class Irreps(tuple):
         else:
             raise ValueError("Normalization needs to be 'norm' or 'component'")
 
-    def __getitem__(self, i):
+    def __getitem__(self, i) -> Union[_MulIr, 'Irreps']:
         x = super().__getitem__(i)
         if isinstance(i, slice):
             return Irreps(x)
@@ -467,7 +468,7 @@ class Irreps(tuple):
 
         Parameters
         ----------
-        ir : `Irrep`
+        ir : `e3nn.o3.Irrep`
 
         Returns
         -------
@@ -505,7 +506,7 @@ class Irreps(tuple):
 
         Returns
         -------
-        `Irreps`
+        `e3nn.o3.Irreps`
 
         Examples
         --------
@@ -533,7 +534,7 @@ class Irreps(tuple):
 
         Returns
         -------
-        `Irreps`
+        `e3nn.o3.Irreps`
 
         Examples
         --------
@@ -550,7 +551,7 @@ class Irreps(tuple):
 
         Returns
         -------
-        irreps : `Irreps`
+        irreps : `e3nn.o3.Irreps`
         p : tuple of int
         inv : tuple of int
 
@@ -635,7 +636,7 @@ class Irreps(tuple):
         `torch.Tensor`
             tensor of shape :math:`(..., \mathrm{dim}, \mathrm{dim})`
         """
-        return self.D_from_angles(*o3.quaternion_to_angles(q), k)
+        return self.D_from_angles(*_rotation.quaternion_to_angles(q), k)
 
     def D_from_matrix(self, R):
         r"""Matrix of the representation
@@ -653,4 +654,4 @@ class Irreps(tuple):
         d = torch.det(R).sign()
         R = d[..., None, None] * R
         k = (1 - d) / 2
-        return self.D_from_angles(*o3.matrix_to_angles(R), k)
+        return self.D_from_angles(*_rotation.matrix_to_angles(R), k)
